==> nautobot_ssot-1.6.3.tar.gz: nautobot_ssot-1.6.3/nautobot_ssot/integrations/device42/diffsync/models/nautobot/assets.py <==
"""DiffSyncModel Asset subclasses for Nautobot Device42 data sync."""

from django.core.exceptions import ValidationError
from django.utils.text import slugify
from nautobot.dcim.models import Device, DeviceType, FrontPort, RearPort
from nautobot_ssot.integrations.device42.constant import PLUGIN_CFG
from nautobot_ssot.integrations.device42.diffsync.models.base.assets import (
    PatchPanel,
    PatchPanelRearPort,
    PatchPanelFrontPort,
)
from nautobot_ssot.integrations.device42.diffsync.models.nautobot.dcim import NautobotRack
from nautobot_ssot.integrations.device42.utils import nautobot


def find_site(diffsync, attrs):
    """Method to determine Site for Patch Panel based upon object attributes."""
    pp_site = False
    try:
        if attrs.get("building"):
            pp_site = diffsync.site_map[slugify(attrs["building"])]
        elif attrs.get("room") and attrs.get("rack"):
            rack = diffsync.get(NautobotRack, {"name": attrs["rack"], "group": attrs["room"]})
            site_name = rack.building
            pp_site = diffsync.site_map[slugify(site_name)]
    except KeyError:
        if diffsync.job.kwargs.get("debug"):
            diffsync.job.log_warning(message=f"Unable to find Site {attrs.get('building')}.")
    return pp_site


class NautobotPatchPanel(PatchPanel):
    """Nautobot Patch Panel model."""

    def find_rack(self, diffsync, building: str, room: str, rack: str):
        """Method to determine Site for Patch Panel based upon object attributes."""
        pp_rack = False
        try:
            if building is not None and room is not None and rack is not None:
                pp_rack = diffsync.rack_map[slugify(building)][slugify(room)][rack]
            elif rack is not None:
                for new_rack in diffsync.objects_to_create["racks"]:
                    if new_rack.name is rack:
                        pp_rack = new_rack.id
        except KeyError:
            if diffsync.job.kwargs.get("debug"):
                diffsync.job.log_warning(message=f"Unable to find Rack using Room {room} & Rack {rack}.")
        return pp_rack

    @classmethod
    def create(cls, diffsync, ids, attrs):
        """Create Patch Panel Device in Nautobot."""
        diffsync.job.log_info(message=f"Creating Patch Panel {ids['name']} Device.")
        try:
            diffsync.device_map[ids["name"]]
        except KeyError:
            pp_site = find_site(diffsync=diffsync, attrs=attrs)
            pp_rack = cls.find_rack(
                cls, diffsync=diffsync, building=attrs.get("building"), room=attrs.get("room"), rack=attrs.get("rack")
            )
            pp_role = nautobot.verify_device_role(diffsync=diffsync, role_name="patch panel")
            if attrs.get("in_service"):
                pp_status = diffsync.status_map["active"]
            else:
                pp_status = diffsync.status_map["offline"]
            patch_panel = Device(
                name=ids["name"],
                status_id=pp_status,
                site_id=pp_site,
                device_type_id=diffsync.devicetype_map[slugify(attrs["model"])],
                device_role_id=pp_role,
                serial=attrs["serial_no"],
            )
            if pp_rack is not False and attrs.get("position") and attrs.get("orientation"):
                patch_panel.rack_id = pp_rack
                patch_panel.position = int(attrs["position"])
                patch_panel.face = attrs["orientation"]
            try:
                diffsync.objects_to_create["devices"].append(patch_panel)
                diffsync.device_map[ids["name"]] = patch_panel.id
                return super().create(ids=ids, diffsync=diffsync, attrs=attrs)
            except ValidationError as err:
                if diffsync.job.kwargs.get("debug"):
                    diffsync.job.log_warning(message=f"Unable to create {ids['name']} patch panel. {err}")
                return None

    def update(self, attrs):
        """Update Patch Panel object in Nautobot."""
        ppanel = Device.objects.get(id=self.uuid)
        self.diffsync.job.log_info(message=f"Updating Patch Panel {ppanel.name} Device.")
        if "in_service" in attrs:
            if attrs["in_service"] is True:
                ppanel.status_id = self.diffsync.status_map["active"]
            else:
                ppanel.status_id = self.diffsync.status_map["offline"]
        if "vendor" in attrs and "model" in attrs:
            ppanel.device_type = DeviceType.objects.get(model=attrs["model"])
        if "orientation" in attrs:
            ppanel.face = attrs["orientation"]
        if "position" in attrs:
            ppanel.position = attrs["position"]
        if "building" in attrs:
            pp_site = find_site(diffsync=self.diffsync, attrs=attrs)
            if pp_site:
                ppanel.site_id = pp_site
        if "room" in attrs or "rack" in attrs:
            if attrs.get("building"):
                _building = attrs["building"]
            else:
                _building = self.building
            if "room" in attrs:
                _room = attrs["room"]
            else:
                _room = self.room
            if "rack" in attrs:
                _rack = attrs["rack"]
            else:
                _rack = self.rack
            pp_rack = self.find_rack(diffsync=self.diffsync, building=_building, room=_room, rack=_rack)
            if pp_rack:
                ppanel.rack_id = pp_rack
                ppanel.face = attrs["orientation"] if attrs.get("orientation") else self.orientation
        if "serial_no" in attrs:
            ppanel.serial = attrs["serial_no"]
        try:
            ppanel.validated_save()
            return super().update(attrs)
        except ValidationError as err:
            self.diffsync.job.log_warning(message=f"Unable to update {self.name} patch panel. {err}")
            return None

    def delete(self):
        """Delete Patch Panel Device object from Nautobot.

        Because a Patch Panel Device has a direct relationship with Ports it can't be deleted before they are.
        The self.diffsync.objects_to_delete dictionary stores all objects for deletion and removes them from Nautobot
        in the correct order. This is used in the Nautobot adapter sync_complete function.
        """
        if PLUGIN_CFG.get("delete_on_sync"):
            super().delete()
            self.diffsync.job.log_info(message=f"Patch panel {self.name} will be deleted.")
            _pp = Device.objects.get(id=self.uuid)
            self.diffsync.objects_to_delete["patchpanel"].append(_pp)  # pylint: disable=protected-access
        return self


class NautobotPatchPanelRearPort(PatchPanelRearPort):
    """Nautobot Patch Panel RearPort model."""

    @classmethod
    def create(cls, diffsync, ids, attrs):
        """Create Patch Panel Port in Nautobot."""
        diffsync.job.log_info(message=f"Creating patch panel port {ids['name']} for {ids['patchpanel']}.")
        try:
            diffsync.rp_map[ids["patchpanel"]][ids["name"]]
        except KeyError:
            rear_port = RearPort(
                name=ids["name"],
                device_id=diffsync.device_map[ids["patchpanel"]],
                type=attrs["port_type"],
                positions=ids["name"],
            )
            try:
                diffsync.objects_to_create["rear_ports"].append(rear_port)
                if ids["patchpanel"] not in diffsync.rp_map:
                    diffsync.rp_map[ids["patchpanel"]] = {}
                diffsync.rp_map[ids["patchpanel"]][ids["name"]] = rear_port.id
                return super().create(ids=ids, diffsync=diffsync, attrs=attrs)
            except ValidationError as err:
                if diffsync.job.kwargs.get("debug"):
                    diffsync.job.log_debug(message=f"Unable to create patch panel {ids['name']}. {err}")
                return None

    def update(self, attrs):
        """Update RearPort object in Nautobot."""
        port = RearPort.objects.get(id=self.uuid)
        self.diffsync.job.log_info(message=f"Updating patch panel port {port.name} for {self.patchpanel}.")
        if "type" in attrs:
            port.type = attrs["type"]
        try:
            port.validated_save()
            return super().update(attrs)
        except ValidationError as err:
            self.diffsync.job.log_warning(message=f"Unable to update {self.name} RearPort. {err}")
            return None

    def delete(self):
        """Delete RearPort object from Nautobot."""
        if PLUGIN_CFG.get("delete_on_sync"):
            super().delete()
            self.diffsync.job.log_info(message=f"RearPort {self.name} for {self.patchpanel} will be deleted.")
            port = RearPort.objects.get(id=self.uuid)
            port.delete()
        return self


class NautobotPatchPanelFrontPort(PatchPanelFrontPort):
    """Nautobot Patch Panel FrontPort model."""

    @classmethod
    def create(cls, diffsync, ids, attrs):
        """Create Patch Panel FrontPort in Nautobot."""
        diffsync.job.log_info(message=f"Creating patch panel front port {ids['name']} for {ids['patchpanel']}.")
        try:
            diffsync.fp_map[ids["patchpanel"]][ids["name"]]
        except KeyError:
            front_port = FrontPort(
                name=ids["name"],
                device_id=diffsync.device_map[ids["patchpanel"]],
                type=attrs["port_type"],
                rear_port_id=diffsync.rp_map[ids["patchpanel"]][ids["name"]],
                rear_port_position=ids["name"],
            )
            try:
                diffsync.objects_to_create["front_ports"].append(front_port)
                if ids["patchpanel"] not in diffsync.fp_map:
                    diffsync.fp_map[ids["patchpanel"]] = {}
                diffsync.fp_map[ids["patchpanel"]][ids["name"]] = front_port.id
                return super().create(ids=ids, diffsync=diffsync, attrs=attrs)
            except ValidationError as err:
                diffsync.job.log_debug(message=f"Unable to create patch panel front port {ids['name']}. {err}")
                return None

    def update(self, attrs):
        """Update FrontPort object in Nautobot."""
        port = FrontPort.objects.get(id=self.uuid)
        self.diffsync.job.log_info(message=f"Updating patch panel front port {self.name} for {self.patchpanel}.")
        if "type" in attrs:
            port.type = attrs["type"]
        try:
            port.validated_save()
            return super().update(attrs)
        except ValidationError as err:
            self.diffsync.job.log_warning(message=f"Unable to update {self.name} FrontPort. {err}")
            return None

    def delete(self):
        """Delete FrontPort object from Nautobot."""
        if PLUGIN_CFG.get("delete_on_sync"):
            super().delete()
            self.diffsync.job.log_info(message=f"FrontPort {self.name} for {self.patchpanel} will be deleted.")
            port = FrontPort.objects.get(id=self.uuid)
            port.delete()
        return self

==> nautobot_ssot-1.6.4.tar.gz: nautobot_ssot-1.6.4/nautobot_ssot/integrations/device42/diffsync/models/nautobot/assets.py <==
"""DiffSyncModel Asset subclasses for Nautobot Device42 data sync."""

from django.core.exceptions import ValidationError
from django.utils.text import slugify
from nautobot.dcim.models import Device, DeviceType, FrontPort, RearPort
from nautobot_ssot.integrations.device42.constant import PLUGIN_CFG
from nautobot_ssot.integrations.device42.diffsync.models.base.assets import (
    PatchPanel,
    PatchPanelRearPort,
    PatchPanelFrontPort,
)
from nautobot_ssot.integrations.device42.diffsync.models.nautobot.dcim import NautobotRack
from nautobot_ssot.integrations.device42.utils import nautobot


def find_site(diffsync, attrs):
    """Method to determine Site for Patch Panel based upon object attributes."""
    pp_site = False
    try:
        if attrs.get("building"):
            pp_site = diffsync.site_map[slugify(attrs["building"])]
        elif attrs.get("room") and attrs.get("rack"):
            rack = diffsync.get(NautobotRack, {"name": attrs["rack"], "group": attrs["room"]})
            site_name = rack.building
            pp_site = diffsync.site_map[slugify(site_name)]
    except KeyError:
        if diffsync.job.kwargs.get("debug"):
            diffsync.job.log_warning(message=f"Unable to find Site {attrs.get('building')}.")
    return pp_site


class NautobotPatchPanel(PatchPanel):
    """Nautobot Patch Panel model."""

    def find_rack(self, diffsync, building: str, room: str, rack: str):
        """Method to determine Site for Patch Panel based upon object attributes."""
        pp_rack = False
        try:
            if building is not None and room is not None and rack is not None:
                pp_rack = diffsync.rack_map[slugify(building)][slugify(room)][rack]
            elif rack is not None:
                for new_rack in diffsync.objects_to_create["racks"]:
                    if new_rack.name is rack:
                        pp_rack = new_rack.id
        except KeyError:
            if diffsync.job.kwargs.get("debug"):
                diffsync.job.log_warning(message=f"Unable to find Rack using Room {room} & Rack {rack}.")
        return pp_rack

    @classmethod
    def create(cls, diffsync, ids, attrs):
        """Create Patch Panel Device in Nautobot."""
        diffsync.job.log_info(message=f"Creating Patch Panel {ids['name']} Device.")
        try:
            diffsync.device_map[ids["name"]]
        except KeyError:
            pp_site = find_site(diffsync=diffsync, attrs=attrs)
            pp_rack = cls.find_rack(
                cls, diffsync=diffsync, building=attrs.get("building"), room=attrs.get("room"), rack=attrs.get("rack")
            )
            pp_role = nautobot.verify_device_role(diffsync=diffsync, role_name="patch panel")
            if attrs.get("in_service"):
                pp_status = diffsync.status_map["active"]
            else:
                pp_status = diffsync.status_map["offline"]
            patch_panel = Device(
                name=ids["name"],
                status_id=pp_status,
                site_id=pp_site,
                device_type_id=diffsync.devicetype_map[slugify(attrs["model"])],
                device_role_id=pp_role,
                serial=attrs["serial_no"],
            )
            if pp_rack is not False and attrs.get("position") and attrs.get("orientation"):
                patch_panel.rack_id = pp_rack
                patch_panel.position = int(attrs["position"])
                patch_panel.face = attrs["orientation"]
            try:
                diffsync.objects_to_create["devices"].append(patch_panel)
                diffsync.device_map[ids["name"]] = patch_panel.id
                return super().create(ids=ids, diffsync=diffsync, attrs=attrs)
            except ValidationError as err:
                if diffsync.job.kwargs.get("debug"):
                    diffsync.job.log_warning(message=f"Unable to create {ids['name']} patch panel. {err}")
                return None

    def update(self, attrs):
        """Update Patch Panel object in Nautobot."""
        ppanel = Device.objects.get(id=self.uuid)
        self.diffsync.job.log_info(message=f"Updating Patch Panel {ppanel.name} Device.")
        if "in_service" in attrs:
            if attrs["in_service"] is True:
                ppanel.status_id = self.diffsync.status_map["active"]
            else:
                ppanel.status_id = self.diffsync.status_map["offline"]
        if "vendor" in attrs and "model" in attrs:
            ppanel.device_type = DeviceType.objects.get(model=attrs["model"])
        if "orientation" in attrs:
            ppanel.face = attrs["orientation"]
        if "position" in attrs:
            ppanel.position = attrs["position"]
        if "building" in attrs:
            pp_site = find_site(diffsync=self.diffsync, attrs=attrs)
            if pp_site:
                ppanel.site_id = pp_site
        if "room" in attrs or "rack" in attrs:
            if attrs.get("building"):
                _building = attrs["building"]
            else:
                _building = self.building
            if "room" in attrs:
                _room = attrs["room"]
            else:
                _room = self.room
            if "rack" in attrs:
                _rack = attrs["rack"]
            else:
                _rack = self.rack
            pp_rack = self.find_rack(diffsync=self.diffsync, building=_building, room=_room, rack=_rack)
            if pp_rack:
                ppanel.rack_id = pp_rack
                ppanel.face = attrs["orientation"] if attrs.get("orientation") else self.orientation
        if "serial_no" in attrs:
            ppanel.serial = attrs["serial_no"]
        try:
            ppanel.validated_save()
            return super().update(attrs)
        except ValidationError as err:
            self.diffsync.job.log_warning(message=f"Unable to update {self.name} patch panel. {err}")
            return None

    def delete(self):
        """Delete Patch Panel Device object from Nautobot.

        Because a Patch Panel Device has a direct relationship with Ports it can't be deleted before they are.
        The self.diffsync.objects_to_delete dictionary stores all objects for deletion and removes them from Nautobot
        in the correct order. This is used in the Nautobot adapter sync_complete function.
        """
        if PLUGIN_CFG.get("device42_delete_on_sync"):
            super().delete()
            self.diffsync.job.log_info(message=f"Patch panel {self.name} will be deleted.")
            _pp = Device.objects.get(id=self.uuid)
            self.diffsync.objects_to_delete["patchpanel"].append(_pp)  # pylint: disable=protected-access
        return self


class NautobotPatchPanelRearPort(PatchPanelRearPort):
    """Nautobot Patch Panel RearPort model."""

    @classmethod
    def create(cls, diffsync, ids, attrs):
        """Create Patch Panel Port in Nautobot."""
        diffsync.job.log_info(message=f"Creating patch panel port {ids['name']} for {ids['patchpanel']}.")
        try:
            diffsync.rp_map[ids["patchpanel"]][ids["name"]]
        except KeyError:
            rear_port = RearPort(
                name=ids["name"],
                device_id=diffsync.device_map[ids["patchpanel"]],
                type=attrs["port_type"],
                positions=ids["name"],
            )
            try:
                diffsync.objects_to_create["rear_ports"].append(rear_port)
                if ids["patchpanel"] not in diffsync.rp_map:
                    diffsync.rp_map[ids["patchpanel"]] = {}
                diffsync.rp_map[ids["patchpanel"]][ids["name"]] = rear_port.id
                return super().create(ids=ids, diffsync=diffsync, attrs=attrs)
            except ValidationError as err:
                if diffsync.job.kwargs.get("debug"):
                    diffsync.job.log_debug(message=f"Unable to create patch panel {ids['name']}. {err}")
                return None

    def update(self, attrs):
        """Update RearPort object in Nautobot."""
        port = RearPort.objects.get(id=self.uuid)
        self.diffsync.job.log_info(message=f"Updating patch panel port {port.name} for {self.patchpanel}.")
        if "type" in attrs:
            port.type = attrs["type"]
        try:
            port.validated_save()
            return super().update(attrs)
        except ValidationError as err:
            self.diffsync.job.log_warning(message=f"Unable to update {self.name} RearPort. {err}")
            return None

    def delete(self):
        """Delete RearPort object from Nautobot."""
        if PLUGIN_CFG.get("device42_delete_on_sync"):
            super().delete()
            self.diffsync.job.log_info(message=f"RearPort {self.name} for {self.patchpanel} will be deleted.")
            port = RearPort.objects.get(id=self.uuid)
            port.delete()
        return self


class NautobotPatchPanelFrontPort(PatchPanelFrontPort):
    """Nautobot Patch Panel FrontPort model."""

    @classmethod
    def create(cls, diffsync, ids, attrs):
        """Create Patch Panel FrontPort in Nautobot."""
        diffsync.job.log_info(message=f"Creating patch panel front port {ids['name']} for {ids['patchpanel']}.")
        try:
            diffsync.fp_map[ids["patchpanel"]][ids["name"]]
        except KeyError:
            front_port = FrontPort(
                name=ids["name"],
                device_id=diffsync.device_map[ids["patchpanel"]],
                type=attrs["port_type"],
                rear_port_id=diffsync.rp_map[ids["patchpanel"]][ids["name"]],
                rear_port_position=ids["name"],
            )
            try:
                diffsync.objects_to_create["front_ports"].append(front_port)
                if ids["patchpanel"] not in diffsync.fp_map:
                    diffsync.fp_map[ids["patchpanel"]] = {}
                diffsync.fp_map[ids["patchpanel"]][ids["name"]] = front_port.id
                return super().create(ids=ids, diffsync=diffsync, attrs=attrs)
            except ValidationError as err:
                diffsync.job.log_debug(message=f"Unable to create patch panel front port {ids['name']}. {err}")
                return None

    def update(self, attrs):
        """Update FrontPort object in Nautobot."""
        port = FrontPort.objects.get(id=self.uuid)
        self.diffsync.job.log_info(message=f"Updating patch panel front port {self.name} for {self.patchpanel}.")
        if "type" in attrs:
            port.type = attrs["type"]
        try:
            port.validated_save()
            return super().update(attrs)
        except ValidationError as err:
            self.diffsync.job.log_warning(message=f"Unable to update {self.name} FrontPort. {err}")
            return None

    def delete(self):
        """Delete FrontPort object from Nautobot."""
        if PLUGIN_CFG.get("device42_delete_on_sync"):
            super().delete()
            self.diffsync.job.log_info(message=f"FrontPort {self.name} for {self.patchpanel} will be deleted.")
            port = FrontPort.objects.get(id=self.uuid)
            port.delete()
        return self
